<?php

namespace FiloBlu\FreeShippingOrderInfo\Observer;

/**
 * Class SetIsFreeShippingCouponObserver
 * @package FiloBlu\FreeShippingOrderInfo\Observer
 */
class SetIsFreeShippingCouponObserver implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var \Magento\SalesRule\Model\RuleFactory
     */
    protected $_ruleFactory;
    /**
     * @var \Magento\Framework\Module\Manager
     */
    protected $_moduleManager;
    /**
     * @var \Psr\Log\LoggerInterface \
     */
    protected $_logger;

    /**
     * SetIsFreeShippingCouponObserver constructor.
     * @param \Magento\SalesRule\Model\RuleFactory $ruleFactory
     * @param \Magento\Framework\Module\Manager $moduleManager
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(\Magento\SalesRule\Model\RuleFactory $ruleFactory,
                                \Magento\Framework\Module\Manager $moduleManager,
                                \Psr\Log\LoggerInterface $logger)
    {
        $this->_logger = $logger;
        $this->_ruleFactory = $ruleFactory;
        $this->_moduleManager = $moduleManager;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @return $this|void
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        try {
            $oadEnabled = $this->_moduleManager->isEnabled('FiloBlu_OrderAdditionalData');
            $order = $observer->getOrder();
            $extensionAttributes = $order->getExtensionAttributes();

            foreach (explode(',', $order->getAppliedRuleIds()) as $ruleId) {
                /** @var Magento\SalesRule\Model\Rule $rule */
                $rule = $this->_ruleFactory->create()->load($ruleId);

                if (!$rule->getRuleId()) {
                    continue;
                }

                $isFreeShipping = $rule->getSimpleFreeShipping();

                if (!$isFreeShipping) {
                    continue;
                }

                $order->setIsFreeShippingCoupon($isFreeShipping);

                if ($oadEnabled) {
                    $extensionAttributes->setIsFreeShippingCoupon($isFreeShipping);
                    $order->setExtensionAttributes($extensionAttributes);
                }

                $order->save();
                return $this;
            }

        }catch (\Exception $e)
        {
            $this->_logger->critical($e->getMessage());
        }

        return $this;
    }
}